
# q-expansion of newform 87.2.a.b, downloaded from the LMFDB on 09 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 87
weight = 2
poly_data = [-1, -4, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0], 1], [[0, 1, 0], 1], [[-3, 0, 1], 1]]

hecke_ring_character_values = None
aps_data = [[1, 0, 1], [-1, 0, 0], [0, -2, 0], [1, 1, -1], [-3, 1, -1], [1, -1, -1], [1, 3, -1], [0, 0, 2], [2, -2, 0], [1, 0, 0], [2, -2, 0], [2, 0, -2], [-2, 4, -4], [0, -4, 4], [-3, -3, 3], [4, 2, 4], [-6, -2, 2], [2, 0, 2], [-1, 3, -3], [-6, -2, -4], [-2, 0, 2], [0, -4, 2], [-2, 2, 2], [-1, 1, 5], [0, 2, -4], [-3, 3, -5], [8, 0, 0], [0, 8, -2], [-5, 1, 1], [9, -5, -1], [2, -6, -2], [-7, 5, 3], [14, -4, 0], [5, 1, -1], [-4, 6, 2], [0, 8, 0], [2, 4, 4], [-6, 6, -10], [4, 4, 8], [8, 2, 2], [6, 6, 0], [-3, -5, -1], [-12, -4, -4], [6, -4, 8], [4, -6, 8], [3, -5, 1], [2, -2, -2], [-5, -5, 1], [-6, -2, -2], [-4, -6, -4], [2, 0, -4], [12, 0, -2], [-9, 9, 1], [-7, -3, -5], [20, 2, -2], [0, 0, -8], [-15, 7, -5], [-14, 2, -6], [3, 9, 5], [6, -12, 8], [20, 0, 4], [-21, 1, -3], [2, 2, 8], [-11, -3, 3], [-3, 7, -5], [-9, -3, -3], [4, -12, 2], [8, 10, -4], [2, -6, -4], [-18, -8, 4], [12, -10, 4], [24, 0, 4], [-16, 0, -4], [14, 0, -4], [0, -8, -6], [-8, -12, 6], [19, 1, 1], [-14, 4, -4], [4, 6, 0], [14, -4, 0], [-8, 0, 2], [14, -8, 8], [4, 4, -4], [0, -6, 0], [27, 3, 1], [15, 3, -3], [1, -5, 3], [5, -9, -1], [14, 0, 0], [3, 3, 9], [-12, -8, 0], [-8, 8, 4], [16, 8, -8], [-8, -12, 4], [9, 13, -13], [13, 5, -5], [-6, -20, 12], [8, -2, -6], [-3, 9, -1], [0, -2, -4], [-15, -3, -1], [10, 4, -8], [-21, -1, -11], [27, 5, -3], [12, 8, -4], [16, 6, 6], [-6, 14, -14], [-4, -2, 8], [21, -11, 15], [-4, 14, 12], [-6, -14, -2], [1, 7, 7], [-14, 16, 4], [0, 0, -2], [3, 11, -7], [-5, -3, 5], [-21, -1, 9], [-2, 10, -4], [-13, 1, -19], [-13, -9, -3], [17, -17, -5], [17, -5, 3], [-7, -1, -1], [-12, 8, -16], [-17, 11, -7], [2, 0, 10], [18, 12, -12], [12, 20, -8], [10, -10, 20], [-16, 2, 6], [-2, 14, 12], [-15, 9, -1], [0, -8, 8], [20, 6, 6], [-20, 6, -16], [-2, 16, -2], [18, -12, -8], [0, 4, 8], [2, 4, -4], [-11, 7, -9], [11, 15, -3], [6, -8, 4], [-8, -24, 16], [16, 12, 4], [-8, 10, -2], [13, 13, 7], [-8, 2, -18], [-8, 2, -4], [34, 6, -2], [-6, -2, -16], [10, -12, 0], [-17, -15, 9], [-16, 4, -16], [15, -1, -11], [34, 6, 6], [5, 5, 11], [-1, -25, 5], [-14, 8, -4], [11, 5, 5], [8, -14, 10], [-35, -7, 7], [12, 10, -14], [-16, 12, -2], [0, -28, 8], [-14, -28, 18], [32, 0, -8], [-23, 9, -13], [0, -6, 6]]
